=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2009-06-30 18:09:07 +0000
+++ drizzled/drizzled.cc	2009-07-10 00:01:50 +0000
@@ -46,6 +46,8 @@
 #include "drizzled/temporal_format.h" /* For init_temporal_formats() */
 #include <drizzled/listen.h>
 
+#include <google/protobuf/stubs/common.h>
+
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -580,6 +582,12 @@
   if (opt_secure_file_priv)
     free(opt_secure_file_priv);
 
+  deinit_temporal_formats();
+
+#if GOOGLE_PROTOBUF_VERSION >= 2001000
+  google::protobuf::ShutdownProtobufLibrary();
+#endif
+
   (void) unlink(pidfile_name);	// This may not always exist
 
   if (print_message && server_start_time)

=== modified file 'drizzled/field.cc'
--- drizzled/field.cc	2009-06-13 21:21:13 +0000
+++ drizzled/field.cc	2009-07-10 00:01:50 +0000
@@ -1209,3 +1209,19 @@
 { 
   return table->isWriteSet(field_index); 
 }
+
+void Field::setReadSet(bool arg)
+{
+  if (arg)
+    table->setReadSet(field_index);
+  else
+    assert(0); // Not completed
+}
+
+void Field::setWriteSet(bool arg)
+{
+  if (arg)
+    table->setWriteSet(field_index);
+  else
+    assert(0); // Not completed
+}

=== modified file 'drizzled/field.h'
--- drizzled/field.h	2009-06-13 21:21:13 +0000
+++ drizzled/field.h	2009-07-10 00:01:50 +0000
@@ -37,6 +37,8 @@
 
 #define DATETIME_DEC                     6
 #define DOUBLE_TO_STRING_CONVERSION_BUFFER_SIZE FLOATING_POINT_BUFFER
+#define ASSERT_COLUMN_MARKED_FOR_READ assert(!table || (table->read_set == NULL || isReadSet()))
+#define ASSERT_COLUMN_MARKED_FOR_WRITE assert(!table || (table->write_set == NULL || isWriteSet()))
 
 const uint32_t max_field_size= (uint32_t) 4294967295U;
 
@@ -741,6 +743,8 @@
 
   bool isReadSet();
   bool isWriteSet();
+  void setReadSet(bool arg= true);
+  void setWriteSet(bool arg= true);
 
 private:
 

=== modified file 'drizzled/field/blob.cc'
--- drizzled/field/blob.cc	2009-06-24 18:16:14 +0000
+++ drizzled/field/blob.cc	2009-07-10 00:01:50 +0000
@@ -201,6 +201,8 @@
   char buff[STRING_BUFFER_USUAL_SIZE];
   String tmpstr(buff,sizeof(buff), &my_charset_bin);
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   if (!length)
   {
     memset(ptr, 0, Field_blob::pack_length());
@@ -269,6 +271,7 @@
 int Field_blob::store(double nr)
 {
   const CHARSET_INFO * const cs=charset();
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   value.set_real(nr, NOT_FIXED_DEC, cs);
   return Field_blob::store(value.ptr(),(uint32_t) value.length(), cs);
 }
@@ -277,6 +280,7 @@
 int Field_blob::store(int64_t nr, bool unsigned_val)
 {
   const CHARSET_INFO * const cs=charset();
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   value.set_int(nr, unsigned_val, cs);
   return Field_blob::store(value.ptr(), (uint32_t) value.length(), cs);
 }
@@ -289,6 +293,8 @@
   uint32_t length;
   const CHARSET_INFO *cs;
 
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   memcpy(&blob,ptr+packlength,sizeof(char*));
   if (!blob)
     return 0.0;
@@ -302,6 +308,9 @@
 {
   int not_used;
   char *blob;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   memcpy(&blob,ptr+packlength,sizeof(char*));
   if (!blob)
     return 0;
@@ -313,6 +322,9 @@
 			    String *val_ptr)
 {
   char *blob;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   memcpy(&blob,ptr+packlength,sizeof(char*));
   if (!blob)
     val_ptr->set("",0,charset());	// A bit safer than ->length(0)
@@ -326,6 +338,9 @@
 {
   const char *blob;
   size_t length;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   memcpy(&blob, ptr+packlength, sizeof(const unsigned char*));
   if (!blob)
   {

=== modified file 'drizzled/field/date.cc'
--- drizzled/field/date.cc	2009-03-08 23:45:12 +0000
+++ drizzled/field/date.cc	2009-07-10 00:01:50 +0000
@@ -93,6 +93,7 @@
    * automatic conversion from the higher-storage DateTime can be used
    * and matches on datetime format strings can occur.
    */
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   drizzled::DateTime temporal;
   if (! temporal.from_string(from, (size_t) len))
   {
@@ -107,6 +108,7 @@
 
 int Field_date::store(double from)
 {
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   if (from < 0.0 || from > 99991231235959.0)
   {
     /* Convert the double to a string using stringstream */
@@ -127,6 +129,7 @@
    * Try to create a DateTime from the supplied integer.  Throw an error
    * if unable to create a valid DateTime.  
    */
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   drizzled::DateTime temporal;
   if (! temporal.from_int64_t(from))
   {
@@ -194,8 +197,13 @@
 
 int64_t Field_date::val_int(void)
 {
-  uint32_t j= uint3korr(ptr);
+  uint32_t j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
+  j= uint3korr(ptr);
   j= (j % 32L)+(j / 32L % 16L)*100L + (j/(16L*32L))*10000L;
+
   return (int64_t) j;
 }
 
@@ -208,6 +216,8 @@
   int part;
   char *pos=(char*) val_buffer->ptr()+10;
 
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   /* Open coded to get more speed */
   *pos--=0;					// End NULL
   part=(int) (tmp & 31);

=== modified file 'drizzled/field/datetime.cc'
--- drizzled/field/datetime.cc	2009-06-30 06:09:40 +0000
+++ drizzled/field/datetime.cc	2009-07-10 00:01:50 +0000
@@ -40,6 +40,7 @@
                           uint32_t len,
                           const CHARSET_INFO * const )
 {
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   /* 
    * Try to create a DateTime from the supplied string.  Throw an error
    * if unable to create a valid DateTime.  
@@ -67,6 +68,7 @@
 
 int Field_datetime::store(double from)
 {
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   if (from < 0.0 || from > 99991231235959.0)
   {
     /* Convert the double to a string using stringstream */
@@ -83,6 +85,7 @@
 
 int Field_datetime::store(int64_t from, bool)
 {
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   /* 
    * Try to create a DateTime from the supplied integer.  Throw an error
    * if unable to create a valid DateTime.  
@@ -162,6 +165,9 @@
 int64_t Field_datetime::val_int(void)
 {
   int64_t j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table && table->s->db_low_byte_first)
     j=sint8korr(ptr);
@@ -179,6 +185,8 @@
   val_buffer->length(drizzled::DateTime::MAX_STRING_LENGTH);
   int64_t tmp;
 
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table && table->s->db_low_byte_first)
     tmp=sint8korr(ptr);

=== modified file 'drizzled/field/decimal.cc'
--- drizzled/field/decimal.cc	2009-06-05 22:51:06 +0000
+++ drizzled/field/decimal.cc	2009-07-10 00:01:50 +0000
@@ -128,6 +128,8 @@
   int err;
   my_decimal decimal_value;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   if ((err= str2my_decimal(E_DEC_FATAL_ERROR &
                            ~(E_DEC_OVERFLOW | E_DEC_BAD_NUM),
                            from, length, charset_arg,
@@ -189,6 +191,8 @@
   my_decimal decimal_value;
   int err;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   err= double2my_decimal(E_DEC_FATAL_ERROR & ~E_DEC_OVERFLOW, nr,
                          &decimal_value);
   if (err)
@@ -211,6 +215,8 @@
   my_decimal decimal_value;
   int err;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   if ((err= int2my_decimal(E_DEC_FATAL_ERROR & ~E_DEC_OVERFLOW,
                            nr, unsigned_val, &decimal_value)))
   {
@@ -245,7 +251,11 @@
 {
   double dbl;
   my_decimal decimal_value;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   my_decimal2double(E_DEC_FATAL_ERROR, val_decimal(&decimal_value), &dbl);
+
   return dbl;
 }
 
@@ -254,13 +264,19 @@
 {
   int64_t i;
   my_decimal decimal_value;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   my_decimal2int(E_DEC_FATAL_ERROR, val_decimal(&decimal_value), false, &i);
+
   return i;
 }
 
 
 my_decimal* Field_new_decimal::val_decimal(my_decimal *decimal_value)
 {
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   binary2my_decimal(E_DEC_FATAL_ERROR, ptr, decimal_value,
                     precision, dec);
   return(decimal_value);
@@ -271,6 +287,9 @@
                                    String *)
 {
   my_decimal decimal_value;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   uint32_t fixed_precision= decimal_precision ? precision : 0;
   my_decimal2string(E_DEC_FATAL_ERROR, val_decimal(&decimal_value),
                     fixed_precision, dec, '0', val_buffer);

=== modified file 'drizzled/field/double.cc'
--- drizzled/field/double.cc	2009-06-26 19:37:44 +0000
+++ drizzled/field/double.cc	2009-07-10 00:01:50 +0000
@@ -39,6 +39,8 @@
   int error;
   char *end;
   double nr= my_strntod(cs,(char*) from, len, &end, &error);
+
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
   if (error || (!len || (((uint32_t) (end-from) != len) && table->in_use->count_cuted_fields)))
   {
     set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN,
@@ -54,6 +56,8 @@
 {
   int error= truncate(&nr, DBL_MAX);
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
   {
@@ -75,6 +79,9 @@
 double Field_double::val_real(void)
 {
   double j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
   {
@@ -90,6 +97,9 @@
 {
   double j;
   int64_t res;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
   {
@@ -129,6 +139,9 @@
 			      String *)
 {
   double nr;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
   {

=== modified file 'drizzled/field/enum.cc'
--- drizzled/field/enum.cc	2009-03-16 15:20:27 +0000
+++ drizzled/field/enum.cc	2009-07-10 00:01:50 +0000
@@ -92,6 +92,8 @@
 {
   uint32_t tmp;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   /* Remove end space */
   length= field_charset->cset->lengthsp(field_charset, from, length);
   tmp= find_type2(typelib, from, length, field_charset);
@@ -135,6 +137,8 @@
  */
 int Field_enum::store(int64_t from, bool)
 {
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   if (from <= 0 || (uint64_t) from > typelib->count)
   {
     /* Convert the integer to a string using stringstream */
@@ -156,6 +160,8 @@
 
 int64_t Field_enum::val_int(void)
 {
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   switch (packlength) {
   case 1:
     return (int64_t) ptr[0];
@@ -219,6 +225,9 @@
 String *Field_enum::val_str(String *, String *val_ptr)
 {
   uint32_t tmp=(uint32_t) Field_enum::val_int();
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   if (!tmp || tmp > typelib->count)
     val_ptr->set("", 0, field_charset);
   else

=== modified file 'drizzled/field/int64_t.cc'
--- drizzled/field/int64_t.cc	2009-06-24 18:16:14 +0000
+++ drizzled/field/int64_t.cc	2009-07-10 00:01:50 +0000
@@ -40,6 +40,8 @@
   char *end;
   uint64_t tmp;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   tmp= cs->cset->strntoull10rnd(cs, from, len, false, &end,&error);
   if (error == MY_ERRNO_ERANGE)
   {
@@ -68,6 +70,8 @@
   int error= 0;
   int64_t res;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   nr= rint(nr);
 
   if (nr <= (double) INT64_MIN)
@@ -102,6 +106,8 @@
 {
   int error= 0;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
   {
@@ -117,6 +123,9 @@
 double Field_int64_t::val_real(void)
 {
   int64_t j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
   {
@@ -133,6 +142,9 @@
 int64_t Field_int64_t::val_int(void)
 {
   int64_t j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
     j=sint8korr(ptr);
@@ -152,6 +164,9 @@
   val_buffer->alloc(mlength);
   char *to=(char*) val_buffer->ptr();
   int64_t j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
     j=sint8korr(ptr);

=== modified file 'drizzled/field/long.cc'
--- drizzled/field/long.cc	2009-06-24 18:16:14 +0000
+++ drizzled/field/long.cc	2009-07-10 00:01:50 +0000
@@ -40,6 +40,8 @@
   int error;
   int64_t rnd;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   error= get_int(cs, from, len, &rnd, UINT32_MAX, INT32_MIN, INT32_MAX);
   store_tmp= (long) rnd;
 #ifdef WORDS_BIGENDIAN
@@ -60,6 +62,8 @@
   int32_t res;
   nr=rint(nr);
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   if (nr < (double) INT32_MIN)
   {
     res=(int32_t) INT32_MIN;
@@ -93,6 +97,8 @@
   int error= 0;
   int32_t res;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   if (nr < 0 && unsigned_val)
     nr= ((int64_t) INT32_MAX) + 1;           // Generate overflow
   if (nr < (int64_t) INT32_MIN)
@@ -126,6 +132,9 @@
 double Field_long::val_real(void)
 {
   int32_t j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
     j=sint4korr(ptr);
@@ -138,6 +147,9 @@
 int64_t Field_long::val_int(void)
 {
   int32_t j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   /* See the comment in Field_long::store(int64_t) */
   assert(table->in_use == current_session);
 #ifdef WORDS_BIGENDIAN
@@ -158,6 +170,9 @@
   val_buffer->alloc(mlength);
   char *to=(char*) val_buffer->ptr();
   int32_t j;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table->s->db_low_byte_first)
     j=sint4korr(ptr);

=== modified file 'drizzled/field/num.cc'
--- drizzled/field/num.cc	2009-06-05 23:18:55 +0000
+++ drizzled/field/num.cc	2009-07-10 00:01:50 +0000
@@ -176,6 +176,7 @@
 my_decimal* Field_num::val_decimal(my_decimal *decimal_value)
 {
   assert(result_type() == INT_RESULT);
+
   int64_t nr= val_int();
   int2my_decimal(E_DEC_FATAL_ERROR, nr, false, decimal_value);
   return decimal_value;

=== modified file 'drizzled/field/real.cc'
--- drizzled/field/real.cc	2009-03-08 23:45:12 +0000
+++ drizzled/field/real.cc	2009-07-10 00:01:50 +0000
@@ -131,6 +131,8 @@
 
 my_decimal *Field_real::val_decimal(my_decimal *decimal_value)
 {
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   double2my_decimal(E_DEC_FATAL_ERROR, val_real(), decimal_value);
   return decimal_value;
 }

=== modified file 'drizzled/field/str.cc'
--- drizzled/field/str.cc	2009-06-05 22:51:06 +0000
+++ drizzled/field/str.cc	2009-07-10 00:01:50 +0000
@@ -127,6 +127,8 @@
   size_t length;
   bool error;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   length= my_gcvt(nr, MY_GCVT_ARG_DOUBLE, local_char_length, buff, &error);
   if (error)
   {

=== modified file 'drizzled/field/timestamp.cc'
--- drizzled/field/timestamp.cc	2009-07-02 07:42:18 +0000
+++ drizzled/field/timestamp.cc	2009-07-10 00:01:50 +0000
@@ -150,6 +150,8 @@
 {
   drizzled::Timestamp temporal;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   if (! temporal.from_string(from, (size_t) len))
   {
     my_error(ER_INVALID_UNIX_TIMESTAMP_VALUE, MYF(ME_FATALERROR), from);
@@ -165,6 +167,8 @@
 
 int Field_timestamp::store(double from)
 {
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   if (from < 0 || from > 99991231235959.0)
   {
     /* Convert the double to a string using stringstream */
@@ -181,6 +185,8 @@
 
 int Field_timestamp::store(int64_t from, bool)
 {
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   /* 
    * Try to create a DateTime from the supplied integer.  Throw an error
    * if unable to create a valid DateTime.  
@@ -213,6 +219,8 @@
 {
   uint32_t temp;
 
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
 #ifdef WORDS_BIGENDIAN
   if (table && table->s->db_low_byte_first)
     temp= uint4korr(ptr);

=== modified file 'drizzled/field/varstring.cc'
--- drizzled/field/varstring.cc	2009-06-13 21:21:13 +0000
+++ drizzled/field/varstring.cc	2009-07-10 00:01:50 +0000
@@ -99,6 +99,8 @@
   const char *cannot_convert_error_pos;
   const char *from_end_pos;
 
+  ASSERT_COLUMN_MARKED_FOR_WRITE;
+
   copy_length= well_formed_copy_nchars(field_charset,
                                        (char*) ptr + length_bytes,
                                        field_length,
@@ -139,7 +141,11 @@
 {
   int not_used;
   char *end_not_used;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   uint32_t length= length_bytes == 1 ? (uint32_t) *ptr : uint2korr(ptr);
+
   return my_strntod(field_charset, (char*) ptr+length_bytes, length,
                     &end_not_used, &not_used);
 }
@@ -149,7 +155,12 @@
 {
   int not_used;
   char *end_not_used;
-  uint32_t length= length_bytes == 1 ? (uint32_t) *ptr : uint2korr(ptr);
+  uint32_t length;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
+  length= length_bytes == 1 ? (uint32_t) *ptr : uint2korr(ptr);
+
   return my_strntoll(field_charset, (char*) ptr+length_bytes, length, 10,
                      &end_not_used, &not_used);
 }
@@ -158,14 +169,23 @@
 				 String *val_ptr)
 {
   uint32_t length=  length_bytes == 1 ? (uint32_t) *ptr : uint2korr(ptr);
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
   val_ptr->set((const char*) ptr+length_bytes, length, field_charset);
+
   return val_ptr;
 }
 
 
 my_decimal *Field_varstring::val_decimal(my_decimal *decimal_value)
 {
-  uint32_t length= length_bytes == 1 ? (uint32_t) *ptr : uint2korr(ptr);
+  uint32_t length;
+
+  ASSERT_COLUMN_MARKED_FOR_READ;
+
+  length= length_bytes == 1 ? (uint32_t) *ptr : uint2korr(ptr);
+
   str2my_decimal(E_DEC_FATAL_ERROR, (char*) ptr+length_bytes, length,
                  charset(), decimal_value);
   return decimal_value;

=== modified file 'drizzled/item/cmpfunc.cc'
--- drizzled/item/cmpfunc.cc	2009-06-24 19:16:57 +0000
+++ drizzled/item/cmpfunc.cc	2009-07-10 00:01:50 +0000
@@ -430,6 +430,8 @@
   Field *field= field_item->field;
   int result= 0;
 
+  field->setWriteSet();
+
   if (!(*item)->with_subselect && (*item)->const_item())
   {
     ulong orig_sql_mode= session->variables.sql_mode;

=== modified file 'drizzled/key.cc'
--- drizzled/key.cc	2009-06-24 17:47:02 +0000
+++ drizzled/key.cc	2009-07-10 00:01:50 +0000
@@ -196,9 +196,13 @@
         restore the part of a record.
         Maybe this branch is to be removed, but now we
         have to ignore GCov compaining.
+
+        This may make more sense once we push down block lengths to the engine (aka partial retrieval).
       */
       uint32_t blob_length= uint2korr(from_key);
       Field_blob *field= (Field_blob*) key_part->field;
+
+      field->setReadSet();
       from_key+= HA_KEY_BLOB_LENGTH;
       key_length-= HA_KEY_BLOB_LENGTH;
       field->set_ptr_offset(to_record - field->table->record[0],
@@ -209,6 +213,9 @@
     {
       Field *field= key_part->field;
       my_ptrdiff_t ptrdiff= to_record - field->table->record[0];
+
+      field->setReadSet();
+      field->setWriteSet();
       field->move_field_offset(ptrdiff);
       key_length-= HA_KEY_BLOB_LENGTH;
       length= min(key_length, key_part->length);
@@ -339,9 +346,10 @@
 	continue;
       }
     }
-    if ((field=key_part->field))
+    if ((field= key_part->field))
     {
       const CHARSET_INFO * const cs= field->charset();
+      field->setReadSet();
       field->val_str(&tmp);
       if (cs->mbmaxlen > 1 &&
           table->field[key_part->fieldnr - 1]->field_length !=
@@ -368,8 +376,6 @@
     else
       to->append(STRING_WITH_LEN("???"));
   }
-
-  return;
 }
 
 

=== modified file 'drizzled/opt_range.cc'
--- drizzled/opt_range.cc	2009-07-02 17:18:18 +0000
+++ drizzled/opt_range.cc	2009-07-10 00:01:50 +0000
@@ -4108,7 +4108,10 @@
     if (arg != field_item)
       ref_tables|= arg->used_tables();
   }
+
   Field *field= field_item->field;
+  field->setWriteSet();
+
   Item_result cmp_type= field->cmp_type();
   if (!((ref_tables | field->table->map) & param_comp))
     ftree= get_func_mm_tree(param, cond_func, field, value, cmp_type, inv);
@@ -4120,6 +4123,8 @@
     while ((item= it++))
     {
       Field *f= item->field;
+      f->setWriteSet();
+
       if (field->eq(f))
         continue;
       if (!((ref_tables | f->table->map) & param_comp))
@@ -4270,6 +4275,8 @@
     while ((field_item= it++))
     {
       Field *field= field_item->field;
+      field->setWriteSet();
+
       Item_result cmp_type= field->cmp_type();
       if (!((ref_tables | field->table->map) & param_comp))
       {

=== modified file 'drizzled/opt_sum.cc'
--- drizzled/opt_sum.cc	2009-06-11 04:17:43 +0000
+++ drizzled/opt_sum.cc	2009-07-10 00:01:50 +0000
@@ -601,6 +601,9 @@
   if (!cond)
     return 1;
   Field *field= field_part->field;
+
+  field->setWriteSet();
+
   if (!(cond->used_tables() & field->table->map))
   {
     /* Condition doesn't restrict the used table */
@@ -831,6 +834,8 @@
 
       /* Check whether the index component is partial */
       Field *part_field= table->field[part->fieldnr-1];
+      part_field->setWriteSet();
+
       if ((part_field->flags & BLOB_FLAG) ||
           part->length < part_field->key_length())
         break;

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2009-07-09 21:14:05 +0000
+++ drizzled/show.cc	2009-07-10 00:01:50 +0000
@@ -1949,6 +1949,7 @@
                                  show_table_list->db_length, false);
 
 
+   table->setWriteSet();
    error= test(schema_table->processTable(session, show_table_list,
                                           table, res, db_name,
                                           table_name));
@@ -2092,6 +2093,8 @@
     res= schema_table->processTable(session, &table_list, table,
                                     res, db_name, table_name);
   }
+  /* For the moment we just set everything to read */
+  table->setReadSet();
 
   release_table_share(share);
 
@@ -2202,6 +2205,7 @@
     goto err;
   }
 
+  table->setWriteSet();
   if (make_db_list(session, db_names, &lookup_field_vals, &with_i_schema))
     goto err;
 
@@ -2483,30 +2487,20 @@
   {
     ptr= show_table->field;
     timestamp_field= show_table->timestamp_field;
-    show_table->use_all_columns();               // Required for default
   }
   else
   {
     ptr= show_table_share->field;
     timestamp_field= show_table_share->timestamp_field;
-    /*
-      read_set may be inited in case of
-      temporary table
-    */
-    if (!show_table->read_set)
-    {
-      /* to satisfy 'field->val_str' ASSERTs */
-      unsigned char *bitmaps;
-      uint32_t bitmap_size= show_table_share->column_bitmap_size;
-      if (!(bitmaps= (unsigned char*) alloc_root(session->mem_root, bitmap_size)))
-        return(0);
-      bitmap_init(&show_table->def_read_set,
-                  (my_bitmap_map*) bitmaps, show_table_share->fields);
-      bitmap_set_all(&show_table->def_read_set);
-      show_table->read_set= &show_table->def_read_set;
-    }
-    show_table->setReadSet();
-  }
+  }
+
+  /* For the moment we just set everything to read */
+  if (!show_table->read_set)
+  {
+    bitmap_set_all(&show_table->def_read_set);
+    show_table->read_set= &show_table->def_read_set;
+  }
+  show_table->use_all_columns();               // Required for default
 
   for (; (field= *ptr) ; ptr++)
   {

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2009-07-03 19:45:58 +0000
+++ drizzled/sql_table.cc	2009-07-10 00:01:50 +0000
@@ -3215,6 +3215,10 @@
       drop_it.remove();
       continue;
     }
+    
+    /* Mark that we will read the field */
+    field->setReadSet();
+
     /* Check if field is changed */
     def_it.rewind();
     while ((def=def_it++))

=== modified file 'drizzled/stored_key.h'
--- drizzled/stored_key.h	2009-06-05 22:22:21 +0000
+++ drizzled/stored_key.h	2009-07-10 00:01:50 +0000
@@ -61,6 +61,8 @@
     else
       to_field= field_arg->new_key_field(session->mem_root, field_arg->table,
                                         ptr, null, 1);
+
+    to_field->setWriteSet();
   }
   virtual ~StoredKey() {}			/** Not actually needed */
   virtual const char *name() const=0;

=== modified file 'drizzled/temporal_format.cc'
--- drizzled/temporal_format.cc	2009-02-28 20:43:31 +0000
+++ drizzled/temporal_format.cc	2009-07-10 00:01:50 +0000
@@ -248,9 +248,10 @@
 , {"^(\\d{1,2})\\.(\\d{1,6})$", 0, 0, 0, 0, 0, 1, 2, 0} /* [S]S.uuuuuu */
 };
 
-std::vector<drizzled::TemporalFormat*> known_datetime_formats;
-std::vector<drizzled::TemporalFormat*> known_date_formats;
-std::vector<drizzled::TemporalFormat*> known_time_formats;
+std::vector<drizzled::TemporalFormat *> known_datetime_formats;
+std::vector<drizzled::TemporalFormat *> known_date_formats;
+std::vector<drizzled::TemporalFormat *> known_time_formats;
+std::vector<drizzled::TemporalFormat *> all_temporal_formats;
 
 /**
  * We allocate and initialize all known date/time formats.
@@ -276,6 +277,12 @@
     tmp->set_second_part_index(current_format_args.second_part_index);
     tmp->set_usecond_part_index(current_format_args.usecond_part_index);
     tmp->set_nsecond_part_index(current_format_args.nsecond_part_index);
+    
+    /* 
+     * We store the pointer in all_temporal_formats because we 
+     * delete pointers from that vector and only that vector
+     */
+    all_temporal_formats.push_back(tmp); 
 
     if (current_format_args.year_part_index > 0) /* A date must have a year */
     {
@@ -289,3 +296,18 @@
   }
   return true;
 }
+
+/** Free all allocated temporal formats */
+void deinit_temporal_formats()
+{
+  std::vector<drizzled::TemporalFormat *>::iterator p= all_temporal_formats.begin();
+  while (p != all_temporal_formats.end())
+  {
+    delete *p;
+    ++p;
+  }
+  known_date_formats.clear();
+  known_datetime_formats.clear();
+  known_time_formats.clear();
+  all_temporal_formats.clear();
+}

=== modified file 'drizzled/temporal_format.h'
--- drizzled/temporal_format.h	2009-02-10 02:29:23 +0000
+++ drizzled/temporal_format.h	2009-07-10 00:01:50 +0000
@@ -145,5 +145,9 @@
  * is initialized on server init.
  */
 bool init_temporal_formats();
+/** 
+ * Frees all memory allocated for temporal format objects
+ */
+void deinit_temporal_formats();
 
 #endif /* DRIZZLED_TEMPORAL_FORMAT_MATCH_H */

=== modified file 'plugin/archive/ha_archive.cc'
--- plugin/archive/ha_archive.cc	2009-06-24 23:17:45 +0000
+++ plugin/archive/ha_archive.cc	2009-07-10 00:01:50 +0000
@@ -1101,6 +1101,10 @@
         if (table->found_next_number_field)
         {
           Field *field= table->found_next_number_field;
+
+          /* Since we will need to use field to translate, we need to flip its read bit */
+          field->setReadSet();
+
           uint64_t auto_value=
             (uint64_t) field->val_int(table->record[0] +
                                        field->offset(table->record[0]));

=== modified file 'plugin/csv/ha_tina.cc'
--- plugin/csv/ha_tina.cc	2009-06-16 01:14:34 +0000
+++ plugin/csv/ha_tina.cc	2009-07-10 00:01:50 +0000
@@ -498,6 +498,12 @@
       (*field)->set_notnull();
     }
 
+    /* 
+      Since we are needing to "translate" the type into a string we 
+      will need to do a val_str(). This would cause an assert() to
+      normally occur since we are onlying "writing" values.
+    */
+    (*field)->setReadSet();
     (*field)->val_str(&attribute,&attribute);
 
     if (was_null)
@@ -606,7 +612,6 @@
   off_t end_offset, curr_offset= current_position;
   int eoln_len;
   int error;
-  bool read_all= false;
 
   free_root(&blobroot, MYF(MY_MARK_BLOCKS_FREE));
 
@@ -623,16 +628,6 @@
 
   memset(buf, 0, table->s->null_bytes);
 
-  /* We need to first check to see if this is a write (rewrite this to something sane that knows if we are doing an update) */
-  for (Field **field=table->field ; *field ; field++)
-  {
-    if ((*field)->isWriteSet())
-    {
-      read_all= true;
-      break;
-    }
-  }
-
   for (Field **field=table->field ; *field ; field++)
   {
     char curr_char;
@@ -698,11 +693,14 @@
       }
     }
 
-    if (read_all || (*field)->isReadSet())
+    if ((*field)->isReadSet() || (*field)->isWriteSet())
     {
+      /* This masks a bug in the logic for a SELECT * */
+      (*field)->setWriteSet();
       if ((*field)->store(buffer.ptr(), buffer.length(), buffer.charset(),
                           CHECK_FIELD_WARN))
         goto err;
+
       if ((*field)->flags & BLOB_FLAG)
       {
         Field_blob *blob= *(Field_blob**) field;

